@@ -14,11 +14,9 @@
 #'  
 #'  However, since the UD dose distribution has some width, and since even the balance point itself is only a close approximation for the actual average of allocated doses, the user's target **does not have to be identical to the balance point.** It only needs to be *"close enough"*.
 #'  
-#'  The `k2targ()` and `g2targ()` utilities are intended for users who already have a specific k-in-a-row or group design in mind, and only want to verify its balance point. The complementary utilities `ktargOptions(), gtargOptions()` provide a broader survey of design-parameter options within user-specified constraints, given a desired target.
+#'  The `k2targ()` and `g2targ()` utilities are intended for users who already have a specific k-in-a-row or group design in mind, and only want to verify its balance point. The complementary utilities `ktargOptions(), gtargOptions()` provide a broader survey of design-parameter options within user-specified constraints, given a desired target. The 0.05 default tolerance level around the target, is what we recommend as *"close enough"*. Otherwise, it is probably better to use biased-coin.
 #'  
-#'  Lastly, `bcoin()` returns the biased-coin probabilities given the user's designated target. In contrast to the two other UDDs described above, the biased-coin design can target any percentile with a precisely matched balance point. That said, k-in-a-row and group UDDs offer some advantages over biased-coin in terms of properties and operational simplicity.
-#'  
-#'  `bcoin()` can return the probability as a decimal (default) or approximate rational fraction. In the latter case, if `nameplate` is set to `TRUE`, you will get the exact *"nameplate"* coin probability \eqn{\Gamma/(1 - \Gamma)}, with \eqn{\Gamma} being the target percentile between 0 and 1. However, the default `nameplate = FALSE` might nudge the coin to yield a balance point somewhat closer to the median. This choice is based upon the theoretical finding that the biased-coin design does tend to concentrate doses a bit further away from the median than the balance point would suggest (Oron and Hoff, 2009). See more information in `bcoin()`'s argument descriptions.
+#'  Lastly, `bcoin()` returns the biased-coin probabilities given the user's designated target. In contrast to the two other UDDs described above, the biased-coin design can target any percentile with a precisely matched balance point. That said, k-in-a-row and group UDDs offer some advantages over biased-coin in terms of performance and operational simplicity. `bcoin()` can return the probability as a decimal, rational fraction, or both (default). 
 #'  
 #' @author Assaf P. Oron \code{<assaf.oron.at.gmail.com>}	
 #'  
@@ -29,20 +27,20 @@
 #' 
 #' @param target the desired target response rate (as a fraction in \eqn{(0,1)}), where relevant.
 #' @param k the number of consecutive identical responses required for dose transitions (k-in-a-row functions only).
-#' @param lowTarget logical, `k2targ()` only: is the design targeting below-median percentiles, with \eqn{k} repeated negative responses needed to level up and only one to level down - or vice versa? Default `FALSE`.
+#' @param lowTarget logical, `k2targ()` only: is the design targeting below-median percentiles, with \eqn{k} repeated negative responses needed to level up and only one to level down - or vice versa? 
 #' @param cohort,lower,upper `g2targ()` only: the cohort (group) size, how many positive responses are allowed for a move upward, and how many are required for a move downward, respectively. For example `cohort=3, lower=0, upper=2` evaluates groups of 3 observations at a time, moves up if none are positive, down if \eqn{>=2} are positive, and repeats the same dose with 1 positive.
 #' @param tolerance 
-#'  - For `ktargOptions(), gtargOptions()`: the half-width of the interval around `target` in which to search for design options. Default 0.1. 
-#'  - For `bcoin()`: the half-width of the interval around 0.5 in which the function recommends to simply use classical UD without a coin, as well as the approximate amount of rounding to the returned coin probability (whether in decimal on rational terms). Default 0.02, and hard-coded to be no less than 0.0001.
+#'  - For `ktargOptions(), gtargOptions()`: the half-width of the interval around `target` in which to search for design options. Default 0.05. 
+#'  - For `bcoin()`: the half-width of the interval around 0.5 in which the function recommends to simply use classical UD without a coin. Default 0.05.
 #' @param maxk `ktargOptions()` only: the maximum value of \eqn{k} to consider.
 #' @param minsize,maxsize `gtargOptions()` only: the minimum and maximum cohort size to consider. `minsize` has to be at least 2 (cohort size 1 is equivalent to classical UD).
-#' @param fraction `bcoin()` only: whether to report the coin probability as a rational rather than decimal fraction. Default `FALSE`. 
-#' @param nameplate `bcoin()` only: in case `fraction = TRUE`, whether to return the "exact" rational probability, or allow some nudging of the resulting balance point towards the median. Default `FALSE`, and moot when `fraction = FALSE`.
+#' @param presentation `bcoin()` only: whether to report the coin probability as a `'decimal'`,  rational `'fraction'`. or `'both'` (default). 
+#' @param digits how many digits to round to? Default 4. Since the functions use R's `round()` utility, trailing zeroes will be truncated.
 #' 
 #' 
 #' @return 
 #'  - `k2targ(), g2targ()`: the official balance point given the user-provided design parameters.
-#'  - `ktargOptions(), gtargOptions()`: a `data.frame` with design parameters and official balance point, for all options that meet user-provided constraints. A printed string provides dose transition rule guidance.
+#'  - `ktargOptions(), gtargOptions()`: a `data.frame` with design parameters and balance points, for all options that meet user-provided constraints. A printed string provides dose transition rule guidance.
 #'  - `bcoin():` a printed string that informs user of the biased-coin design rules, including the 'coin' probability in its user-chosen format (decimal or fraction). In case the user-desired target is 0.5 or very close to it, the string will inform user that they are better off just using classical UDD without a coin.
 #'  
 #'  
@@ -57,38 +55,42 @@
 
 #' @export
 
-k2targ<-function(k, lowTarget=FALSE)
+k2targ<-function(k, lowTarget, digits = 4)
 {
   checkNatural(k, parname = 'k', toolarge = 30)  
   tmp = 0.5 ^ (1/k)
-  if(lowTarget) return(1-tmp)
-  tmp
+  if(lowTarget) return(round(1-tmp, digits) )
+  round(tmp, digits)
 }
 
 #' @rdname k2targ
 #' @export
 
-ktargOptions<-function(target, tolerance = 0.1, maxk = 20)
+ktargOptions<-function(target, tolerance = 0.05, maxk = 20, digits = 4)
 {
   if(length(target) > 1) stop("target must be a single number between 0 and 1.\n")
   checkTarget(target)
   checkTarget(target - tolerance, tname = "target minus tolerance")
-  
+
+# For simplicity, doing the above-median arithmetic
   hi = (target>=0.5) 
   if(!hi) target = 1-target
   
-  hiend = k2targ(maxk)
+  hiend = k2targ(maxk, lowTarget = FALSE)
   
   klo = -1 / log2( max(0.5, target - tolerance) )
   khi = -1 / log2( min(hiend, target + tolerance) )
+  
+#  return(c(klo, khi))
   krange = ceiling(klo):floor(khi)
+  if(any(diff(krange) < 0 ) ) return("No k options within target range; please increase tolerance band or use biased-coin.")
   krange = sort( krange[krange > 0 & krange <= maxk] )
   
-  message("For the following targets, k ", ifelse(hi, "positive", "negative"),
-      " responses are needed for a move ", ifelse(hi, "down.\n", "up.\n"),
+  message("For the following targets, k consecutive ", ifelse(hi, "positive", "negative"),
+      " responses are needed for a move ", ifelse(hi, "DOWN.\n", "UP.\n"),
       "Only one ", ifelse(hi, "negative", "positive"), " response is needed for the opposite move.\n")
   
-  data.frame(k = krange, BalancePoint = k2targ(krange, lowTarget = !hi) )
+  data.frame(k = krange, BalancePoint = k2targ(krange, lowTarget = !hi, digits = digits) )
 }
 
 
@@ -99,14 +101,14 @@ ktargOptions<-function(target, tolerance = 0.1, maxk = 20)
 #' @export
 
 
-g2targ<-function(cohort, lower, upper)
+g2targ<-function(cohort, lower, upper, digits = 4)
 {
 checkNatural(c(cohort, lower+1, upper), parname = 'cohort, lower+1, upper', toolarge = 50)  
 if(cohort<upper || upper<=lower) stop('Order must be lower < upper <= cohort.\n')
 
 if(upper + lower == cohort) return(0.5)
   
-uniroot(f=function(x, k, u, l) {pbinom(q=l, size=k, prob=x) + (pbinom(q=u-1, size=k, prob=x) - 1)}, interval=0:1, k=cohort, u=upper, l=lower)$root
+round(uniroot(f=function(x, k, u, l) {pbinom(q=l, size=k, prob=x) + (pbinom(q=u-1, size=k, prob=x) - 1)}, interval=0:1, k=cohort, u=upper, l=lower)$root, digits)
 }
 
 # uniroot(f=function(x,kay,you,ell,bee1,bee2) {bee1*pbinom(q=ell,size=kay,prob=x)+bee2*(pbinom(q=you-1,size=kay,prob=x)-1)},interval=0:1,kay=k,you=u,ell=l,bee1=b1,bee2=b2)$root
@@ -115,7 +117,7 @@ uniroot(f=function(x, k, u, l) {pbinom(q=l, size=k, prob=x) + (pbinom(q=u-1, siz
 #' @export
 
 # Vals
-gtargOptions<-function(target, minsize = 2, maxsize = 6, tolerance = 0.1)
+gtargOptions<-function(target, minsize = 2, maxsize = 6, tolerance = 0.05, digits = 4)
 {
 if(maxsize<minsize) stop("'maxsize' cannot be smaller than 'minsize'.\n")
 checkNatural(c(maxsize-1, minsize-1), parname = 'Cohort size less 1', toolarge = 50)  
@@ -136,14 +138,14 @@ for(k in minsize:maxsize)
     u = ucand
    if(ucand > l) for(u in seq(ucand, l+1, -1))
     {
-      bal = g2targ(cohort=k, lower=l, upper=u) 
+      bal = g2targ(cohort=k, lower=l, upper=u, digits = digits) 
       if(bal < target-tolerance) break
       if(bal > target+tolerance) next
       dtmp = rbind(dtmp, data.frame( Cohort=k, Lower=l, Upper=u, BalancePoint=bal ) )
     }
     if(ucand < k) for(u in seq(max(l+1,ucand+1), k, 1))
     {
-      bal = g2targ(cohort=k, lower=l, upper=u) 
+      bal = g2targ(cohort=k, lower=l, upper=u, digits = digits) 
       if(bal > target+tolerance) break
       if(bal < target-tolerance) next
       dtmp = rbind(dtmp, data.frame( Cohort=k, Lower=l, Upper=u, BalancePoint=bal ) )
@@ -152,9 +154,11 @@ for(k in minsize:maxsize)
   }
 }
 
-message("For each design, if positive responses <= Lower, move up\n")
-message("                 if positive responses >= Upper, move down\n")
-message("  otherwise repeat same dose (relevant only when Upper - Lower > 1).\n")
+if(nrow(dout) == 0) return("No Group UDD options within target range; please increase tolerance band or use biased-coin.")
+
+message("For each design, if positive responses <= Lower, move UP.\n")
+message("                 if positive responses >= Upper, move DOWN.\n")
+message("  otherwise, REPEAT same dose (relevant only when Upper - Lower > 1).\n")
 
 return(dout)
 }
@@ -164,15 +168,18 @@ return(dout)
 #' @rdname k2targ
 #' @export
 
-bcoin <- function(target, fraction = FALSE, nameplate = FALSE, tolerance = 0.02)
+bcoin <- function(target, presentation = 'both', digits = 4, tolerance = 0.05)
 {
-requireNamespace('numbers')
+requireNamespace('MASS')
   
 ## Validations
 checkTarget(target)
 checkTarget(tolerance, tname = "'tolerance'")
 if(tolerance >= min(target, 1 - target)/2) stop("'tolerance' is set too large.\n")
 if(tolerance < 1e-4) stop("'tolerance' is set unrealistically small. Change it to 1e-4 or more.\n")
+allowedprez = c('decimal', 'fraction', 'both') 
+if(!(presentation %in% allowedprez) ) 
+  stop("Presentation must be one of ", paste(allowedprez, collapse = ', ') )
 
 ### Targets too close to 0.5:
 
@@ -185,43 +192,31 @@ if(target >= 0.5 - tolerance && target <= 0.5 + tolerance)
 } else if(target < 0.5) { 
   
   coin = target / (1-target)
-  cout = round(coin, digits = ceiling(-log10(tolerance)))
-  if(fraction) 
-# Find rational coin fraction, erring upwards b/c at <0.5 BCD biases a bit down
-  {
-    coinFrac = numbers::ratFarey(coin, n = round(1/tolerance), upper = TRUE) 
-    if(nameplate)
+  if(presentation == 'decimal') cout = round(coin, digits) else
     {
-      frac2 = numbers::ratFarey(coin, n = round(1/tolerance), upper = FALSE) 
-      if(abs(coin - frac2[1]/frac2[2]) < abs(coin - coinFrac[1]/coinFrac[2]))
-        coinFrac = frac2
+      frac = MASS::fractions(coin, cycles = 4)
+      cout = frac
+      if(presentation == 'both') cout = paste(cout, '  (', round(coin, digits), ')', sep='' )
     }
-    cout = paste(coinFrac[1], '/', coinFrac[2])
-  }
-  message(paste("After positive response, move DOWN.
-After negative response, 'toss a COIN':
-   - with probability of approximately", cout, 'move UP
+  
+  message(paste("After a positive response, move DOWN.
+After a negative response, 'toss a Coin':
+   - with probability of", cout, 'move UP
    - Otherwise REPEAT same dose.\n') )
   
 } else if(target > 0.5) { 
   
   coin = (1-target) / target 
-  cout = round(coin, digits = ceiling(-log10(tolerance)))
-  if(fraction) 
-    # Find rational coin fraction, erring upwards b/c at <0.5 BCD biases a bit down
+  if(presentation == 'decimal') cout = round(coin, digits) else
   {
-    coinFrac = numbers::ratFarey(coin, n = round(1/tolerance), upper = FALSE) 
-    if(nameplate)
-    {
-      frac2 = numbers::ratFarey(coin, n = round(1/tolerance), upper = TRUE) 
-      if(abs(coin - frac2[1]/frac2[2]) < abs(coin - coinFrac[1]/coinFrac[2]))
-        coinFrac = frac2
-    }
-    cout = paste(coinFrac[1], '/', coinFrac[2])
+    frac = MASS::fractions(coin, cycles = 4)
+    cout = frac
+    if(presentation == 'both') cout = paste(cout, '  (', round(coin, digits), ')', sep='' )
   }
-  message(paste("After negative response, move UP.
-After positive response, 'toss a COIN':
-   - with probability of approximately", cout, 'move DOWN
+  
+  message(paste("After a negative response, move UP.
+After a positive response, 'toss a Coin':
+   - with probability of", cout, 'move DOWN
    - Otherwise REPEAT same dose.\n') )
 }  
   
@@ -242,7 +237,7 @@ After positive response, 'toss a COIN':
 #' @param k (`checkNatural()` only) input number to check whether it's a natural number 
 #' @param toolarge (`checkNatural()` only) what number would be considered too large to be realistic?
 #' @param x (`checkDose()` only) input object to be verified as valid dose values
-#' @param maxfrac (`checkDose()` only) maximum number of unique values (as fraction of sample size) considered realistic for up-and-down data. Default $0.9n.$ Function also gives a warning if number exceeds $n/2$, since typically this suggests the effective sample size around target is too small.
+#' @param maxfrac (`checkDose()` only) maximum number of unique values (as fraction of sample size) considered realistic for up-and-down data. Default \eqn{0.9n}. Function also gives a warning if number exceeds \eqn{n/2}, since typically this suggests the effective sample size around target is too small.
 #' @param y (`checkResponse()` only) input object to be verified as valid response values (`TRUE/FALSE or 0/1)
 #' @param flatOK logical (`checkCDF()` only) if the CDF is completely flat, should function issue a warning (`TRUE`, default) - or stop with an error (`FALSE`)?
 #' 
